@@ -14,6 +14,7 @@ pub struct Transfer {
 }
 
 /// EVM context handler.
+#[auto_impl::auto_impl(&mut, Box)]
 pub trait Handler {
 	/// Type of `CREATE` interrupt.
 	type CreateInterrupt;
@@ -62,6 +63,13 @@ pub trait Handler {
 	fn exists(&self, address: H160) -> bool;
 	/// Check whether an address has already been deleted.
 	fn deleted(&self, address: H160) -> bool;
+	/// Checks if the address or (address, index) pair has been previously accessed
+	/// (or set in `accessed_addresses` / `accessed_storage_keys` via an access list
+	/// transaction).
+	/// References:
+	/// * https://eips.ethereum.org/EIPS/eip-2929
+	/// * https://eips.ethereum.org/EIPS/eip-2930
+	fn is_cold(&self, address: H160, index: Option<H256>) -> bool;
 
 	/// Set storage value of address at index.
 	fn set_storage(&mut self, address: H160, index: H256, value: H256) -> Result<(), ExitError>;
