@@ -13,7 +13,7 @@ pub mod tracing;
 macro_rules! event {
 	($x:expr) => {
 		use crate::tracing::Event::*;
-		$x.emit();
+		crate::tracing::with(|listener| listener.event($x));
 	};
 }
 
@@ -177,6 +177,8 @@ pub struct Config {
 	pub gas_balance: u64,
 	/// Gas paid for SLOAD opcode.
 	pub gas_sload: u64,
+	/// Gas paid for cold SLOAD opcode.
+	pub gas_sload_cold: u64,
 	/// Gas paid for SUICIDE opcode.
 	pub gas_suicide: u64,
 	/// Gas paid for SUICIDE opcode when it hits a new account.
@@ -193,10 +195,20 @@ pub struct Config {
 	pub gas_transaction_zero_data: u64,
 	/// Gas paid for non-zero data in a transaction.
 	pub gas_transaction_non_zero_data: u64,
+	/// Gas paid per address in transaction access list (see EIP-2930).
+	pub gas_access_list_address: u64,
+	/// Gas paid per storage key in transaction access list (see EIP-2930).
+	pub gas_access_list_storage_key: u64,
+	/// Gas paid for accessing cold account.
+	pub gas_account_access_cold: u64,
+	/// Gas paid for accessing ready storage.
+	pub gas_storage_read_warm: u64,
 	/// EIP-1283.
 	pub sstore_gas_metering: bool,
 	/// EIP-1706.
 	pub sstore_revert_under_stipend: bool,
+	/// EIP-2929
+	pub increase_state_access_gas: bool,
 	/// Whether to throw out of gas error when
 	/// CALL/CALLCODE/DELEGATECALL requires more than maximum amount
 	/// of gas.
@@ -245,6 +257,7 @@ impl Config {
 			gas_ext_code_hash: 20,
 			gas_balance: 20,
 			gas_sload: 50,
+			gas_sload_cold: 0,
 			gas_sstore_set: 20000,
 			gas_sstore_reset: 5000,
 			refund_sstore_clears: 15000,
@@ -256,8 +269,13 @@ impl Config {
 			gas_transaction_call: 21000,
 			gas_transaction_zero_data: 4,
 			gas_transaction_non_zero_data: 68,
+			gas_access_list_address: 0,
+			gas_access_list_storage_key: 0,
+			gas_account_access_cold: 0,
+			gas_storage_read_warm: 0,
 			sstore_gas_metering: false,
 			sstore_revert_under_stipend: false,
+			increase_state_access_gas: false,
 			err_on_call_with_more_gas: true,
 			empty_considered_exists: true,
 			create_increase_nonce: false,
@@ -286,6 +304,7 @@ impl Config {
 			gas_ext_code_hash: 700,
 			gas_balance: 700,
 			gas_sload: 800,
+			gas_sload_cold: 0,
 			gas_sstore_set: 20000,
 			gas_sstore_reset: 5000,
 			refund_sstore_clears: 15000,
@@ -297,8 +316,60 @@ impl Config {
 			gas_transaction_call: 21000,
 			gas_transaction_zero_data: 4,
 			gas_transaction_non_zero_data: 16,
+			gas_access_list_address: 0,
+			gas_access_list_storage_key: 0,
+			gas_account_access_cold: 0,
+			gas_storage_read_warm: 0,
 			sstore_gas_metering: true,
 			sstore_revert_under_stipend: true,
+			increase_state_access_gas: false,
+			err_on_call_with_more_gas: false,
+			empty_considered_exists: false,
+			create_increase_nonce: true,
+			call_l64_after_gas: true,
+			stack_limit: 1024,
+			memory_limit: usize::MAX,
+			call_stack_limit: 1024,
+			create_contract_limit: Some(0x6000),
+			call_stipend: 2300,
+			has_delegate_call: true,
+			has_create2: true,
+			has_revert: true,
+			has_return_data: true,
+			has_bitwise_shifting: true,
+			has_chain_id: true,
+			has_self_balance: true,
+			has_ext_code_hash: true,
+			estimate: false,
+		}
+	}
+
+	/// Berlin hard fork configuration.
+	pub const fn berlin() -> Config {
+		Config {
+			gas_ext_code: 0,
+			gas_ext_code_hash: 0,
+			gas_balance: 0,
+			gas_sload: 0,
+			gas_sload_cold: 2100,
+			gas_sstore_set: 20000,
+			gas_sstore_reset: 5000,
+			refund_sstore_clears: 15000,
+			gas_suicide: 5000,
+			gas_suicide_new_account: 25000,
+			gas_call: 0,
+			gas_expbyte: 50,
+			gas_transaction_create: 53000,
+			gas_transaction_call: 21000,
+			gas_transaction_zero_data: 4,
+			gas_transaction_non_zero_data: 16,
+			gas_access_list_address: 2400,
+			gas_access_list_storage_key: 1900,
+			gas_account_access_cold: 2600,
+			gas_storage_read_warm: 100,
+			sstore_gas_metering: true,
+			sstore_revert_under_stipend: true,
+			increase_state_access_gas: true,
 			err_on_call_with_more_gas: false,
 			empty_considered_exists: false,
 			create_increase_nonce: true,
